#!/usr/bin/python
from scipy.interpolate import interp1d
from scipy.special import erfcinv
import matplotlib
matplotlib.use("Agg")
from matplotlib import rc
matplotlib.rc('text', usetex=True)
# matplotlib.rc('font', family='sans')
import matplotlib.pyplot as plt
import numpy as np
import matplotlib.font_manager as ftmgr
import detgeom

# gstHr = np.linspace(0,24,100000)
gstHr = np.arange(0,24,.1)

SN1987A_raHr = 5. + 35./60. + 28.03/3600.
SN1987A_decDeg = -(69. + 16./60. + 11.79/3600.)
SN1987A = detgeom.SkyPosition(SN1987A_decDeg,SN1987A_raHr)
LHOdet = detgeom.LHO()
LLOdet = detgeom.LLO()
SN1987A_aL = np.array([ LLOdet.amCoefficients(SN1987A,g)[0] for g in gstHr ])
SN1987A_bL = np.array([ LLOdet.amCoefficients(SN1987A,g)[1] for g in gstHr ])
SN1987A_aH = np.array([ LHOdet.amCoefficients(SN1987A,g)[0] for g in gstHr ])
SN1987A_bH = np.array([ LHOdet.amCoefficients(SN1987A,g)[1] for g in gstHr ])
SN1987A_GIJ = 0.1 * (SN1987A_aH*SN1987A_aL + SN1987A_bH*SN1987A_bL)
SN1987A_GIJrms = np.sqrt(np.mean(SN1987A_GIJ**2))
print 'SN 1987A:'
print '    declination is %.4f degrees = %.5f radians'\
    % (SN1987A_decDeg,SN1987A_decDeg*np.pi/180.)
print '    right ascension is %.4f hours = %.4f degrees = %.5f radians'\
    % (SN1987A_raHr,SN1987A_raHr*180./12.,SN1987A_raHr*np.pi/12.)
print '    <|GIJ|^2>^-1/4 = %.4f' % (1/np.sqrt(SN1987A_GIJrms))

ScoX1_raHr = 16. + 19./60. + 55.07/3600.
ScoX1_decDeg = -(15. + 38./60. + 24.8/3600.)
ScoX1 = detgeom.SkyPosition(ScoX1_decDeg,ScoX1_raHr)
LHOdet = detgeom.LHO()
LLOdet = detgeom.LLO()
ScoX1_aL = np.array([ LLOdet.amCoefficients(ScoX1,g)[0] for g in gstHr ])
ScoX1_bL = np.array([ LLOdet.amCoefficients(ScoX1,g)[1] for g in gstHr ])
ScoX1_aH = np.array([ LHOdet.amCoefficients(ScoX1,g)[0] for g in gstHr ])
ScoX1_bH = np.array([ LHOdet.amCoefficients(ScoX1,g)[1] for g in gstHr ])
ScoX1_GIJ = 0.1 * (ScoX1_aH*ScoX1_aL + ScoX1_bH*ScoX1_bL)
ScoX1_GIJrms = np.sqrt(np.mean(ScoX1_GIJ**2))
# print 1/np.sqrt(ScoX1_GIJrms)

decDeg = np.linspace(-90.,90.,200)
raHr = 0. * decDeg
skyPos = detgeom.SkyPosition(decDeg,raHr)
aL = np.array([ LLOdet.amCoefficients(skyPos,g)[0] for g in gstHr ])
bL = np.array([ LLOdet.amCoefficients(skyPos,g)[1] for g in gstHr ])
aH = np.array([ LHOdet.amCoefficients(skyPos,g)[0] for g in gstHr ])
bH = np.array([ LHOdet.amCoefficients(skyPos,g)[1] for g in gstHr ])
GIJ = 0.1 * (aH*aL + bH*bL)
GIJrms = np.sqrt(np.mean(GIJ**2,axis=0))

fig = plt.figure()
fig.set_size_inches(5*.8,3*.8)
ax = fig.add_subplot(111)

ymin = 0.
ymax = 6.
ax.plot(decDeg,1./np.sqrt(GIJrms),'k-')
ax.plot([SN1987A_decDeg,SN1987A_decDeg],[ymin,ymax],'b-')
ax.text(SN1987A_decDeg+7.,2.,'SN1987A',rotation='vertical',color='b')
ax.plot([ScoX1_decDeg,ScoX1_decDeg],[ymin,ymax],'r-')
ax.text(ScoX1_decDeg+7.,2.,'ScoX1',rotation='vertical',color='r')
ax.set_xlabel('Declination (degrees)')
ax.set_ylabel(r'$\langle|\tilde{\cal G}_{IJ}|^2\rangle^{-1/4}$')
ax.set_xlim([90,-90])
ax.set_ylim([ymin,ymax])
ax.set_xticks(range(-90,91,30))
ax.grid(True)
fig.savefig('Gfactor.eps',bbox_inches='tight')

f_LIGO,A_LIGO = np.loadtxt('SRD_strain_4k.txt',unpack=True)

fig = plt.figure()
fig.set_size_inches(5*.8,3*.8)
ax = fig.add_subplot(111)

# Use cubic interpolation to get a finer resolution; note the range of
# the SRD data (logarithmically sampled) is too big and confuses
# scipy's interpolation, so we limit it to frequency indices corresponding to
# 100-300 Hz plus one extra point outside that range:
f_fine = np.linspace(100,300,1000)
A_LIGO_fcn = interp1d(f_LIGO[48:61],A_LIGO[48:61],kind='cubic')
A_fine = A_LIGO_fcn(f_fine)

Tsft = 1800. # SFT duration in seconds
false_alarm = 0.05
false_dismissal = 0.05
stats = erfcinv(2*false_alarm) + erfcinv(2*false_dismissal)
Npairs = 1e5
prefactor = np.sqrt(stats/(2*SN1987A_GIJrms*np.sqrt(Npairs)*Tsft))
print '    h_th = %.2e * ASD' % prefactor

ax.plot(f_fine,prefactor*A_fine,'b-',label='sensitivity')

ax.plot([100,300],[3.39e-25,3.39e-25],'r-',label='age-based limit')

# This is the uninterpolated data:
# ax.plot(f_LIGO,7.48e-3*A_LIGO,'rx')

ax.set_title('SN1987A')
ax.set_xlabel('Frequency (Hz)')
ax.set_ylabel(r'$h_{\textrm{th}}$ (strain)')

ax.set_xlim([100,300])
# ax.set_ylim([2.0e-25,3.4e-25])
ax.set_ylim([1.75e-25,3.45e-25])
ax.set_yticks(np.arange(1.8e-25,3.45e-25,0.2e-25))
# ax.grid(True)
ax.legend()

fig.savefig('sens_SN1987A.eps',bbox_inches='tight')
